#!/usr/bin/perl -w
use strict;

# Returns text of img tag with URL for pathway 
# compounds blue line on red bg (circles filled red)
# e.g. in response to KeggAjax.js on same server to replace 'working ani gif'
# Original version (kegg.cgi) returned html, but 15 sec before img generated
# -T removed as caused error when run from servlet
# DPL 10.06.2010

use CGI qw(:standard);
use SOAP::Lite;

# Get parameters for map and construct
my $path = param('path');	# id of path (e.g. 00260) - must handle as a string
my $org = param('org');					# id of organism (e.g. eco)
my $cmpdNames = param('compounds');		# comma separated list of compounds

# comma-sep list of states of compounds: -1 = decrease, 0 = no change, +1
my $states = param('state');	

my $index = param('index');				# loop integer to identify image map

# construct $map argument for service method parameter
my $prefix = "path:";
my $map = $prefix.$org.$path;

# generate array of compounds from comma-sep string
my @cmpd = split (/,/, $cmpdNames);
my $num = @cmpd; 	# size of this array

# generate array of states from comma-sep string
my @state = split (/,/, $states);

# set up colours
my $red3 = '#990000';			# fg decrease level -3
my $red2 = '#ff0000';			# fg decrease level -2
my $red1 = '#ffbad3';			# fg decrease level -1
my $yellow = '#ffff00';			# fg no change
my $blue1 = '#78f4fc';			# fg increase level +1
my $blue2 = '#3b82ff';			# fg increase level +2
my $blue3 = '#2d10db';			# fg increase level +3
my $black = '#000000';			# bg (stroke)

# construct array of colours of the same length as compound array for fill
my @colours;
for (my $i=0; $i<$num; $i++)
{
	if($state[$i] == 3)
	{
		$colours[$i] = $blue3;
	}
	elsif($state[$i] == 2)
	{
		$colours[$i] = $blue2;	
	}
	elsif($state[$i] == 1)
	{
		$colours[$i] = $blue1;	
	}
	elsif($state[$i] == -1)
	{
		$colours[$i] = $red1;	
	}
	elsif($state[$i] == -2)
	{
		$colours[$i] = $red2;	
	}
	elsif($state[$i] == -3)
	{
		$colours[$i] = $red3;	
	}
	else
	{
		$colours[$i] = $yellow;	
	}
}

# construct array of blacks of the same length as compound array for line
my @blacks;
for (my $i=0; $i<$num; $i++)
{
	$blacks[$i] = $black;
}

# Construct array arguments for soap service
my $obj_list = SOAP::Data->type(array => [@cmpd]);
my $fg_list = SOAP::Data->type(array => [@blacks]);
my $bg_list = SOAP::Data->type(array => [@colours]);

# Construct soap service
my $wsdl = 'http://soap.genome.jp/KEGG.wsdl';
my $serv = SOAP::Lite -> service($wsdl);

# Use Kegg method to obtain map
my $result = $serv -> color_pathway_by_objects($map, $obj_list, $fg_list, $bg_list);

print header(-type => "text/plain");

# Return Content as text
print "<img src=\"$result\" alt=\"map\" usemap=\"#keggmap$index\" />";	# link to the generated image




