/*
Class holds details for a Mass Spectrometry Adduct
DPL 31.07.2011
 */

public class Adduct 
{
	int id;					// unique identifier
	double adductMass;		// mass of adduct (can be -ve)
	double factor;			// multiplier for metabolite	
	boolean positive;		// whether generated in +ve or -ve mode
	boolean restricted;		// whether member of restricted list of adducts
	String descriptor;		// brief descriptor for output
	String longDescriptor;	// reference descriptor
	boolean selected;		// flag for user selection

	public Adduct(int id, double adductMass, double factor, boolean positive, 
					boolean restricted, String descriptor, String longDescriptor) 
	{
		this.id = id;
		this.adductMass = adductMass;
		this.factor = factor;
		this.positive = positive;
		this.restricted = restricted;
		this.descriptor = descriptor;
		this.longDescriptor = longDescriptor;
		selected = false;
	}
	
	// Given a metabolite mass returns m/z with adduct
	public double makeMZ(double mass)
	{
		return mass*factor + adductMass;
	}

	// -- accessor methods ---//
	public void setSelected(boolean sel)
	{
		selected = sel;
	}
	
	public int getID()
	{
		return id;
	}

	public double getMass()
	{
		return adductMass;
	}
	
	public double getFactor()
	{
		return factor;
	}

	public boolean isPositive() 
	{
		return positive;
	}
	
	public boolean isRestricted() 
	{
		return restricted;
	}
	
	public String getDescriptor()
	{
		return descriptor;
	}
	
	public String getLongDescriptor()
	{
		return longDescriptor;
	}
	
	public boolean isSelected()
	{
		return selected;
	}
}