/*
DBQuery
Class with static methods to provide access to SQL query strings
DPL 30.01.11
 */

public class DBQuery 
{
	final static String name0 = "THEORO_METABOLITES_ORG";
	final static String name1 = "THEORO_METABOLITES_ALL";			
	final static String name2 = "ORGANISM_ID";

	// selects details of all metabolites present in a specific organism (according to Kegg DB)
	final static String query0 = "SELECT DISTINCT mp.MapName, mp.MapID, m.CompoundName, m.Formula, m.FormDegen, m.MIM, m.CompoundID, m.MetacycUID, "
												+ "mm.GlyphShape, mm.GlyphX, mm.GlyphY, mm.GlyphWidth, mm.GlyphHeight "
			+ "FROM Map mp, Metabolite m, MetaboliteOfMap mm, MapOfOrg mo, Organism o "
			+ "WHERE mm.MapID = mp.MapID  "
			+ "AND mm.CompoundID = m.CompoundID "
			+ "AND mp.MapID = mo.MapID "
			+ "AND mo.OrganismID = o.OrganismID "
			+ "AND o.OrganismName = ? "
			+ "ORDER BY mp.MapName";

	// selects details of all metabolites present in MetabDB (according to Kegg DB)
	final static String query1 = "SELECT DISTINCT mp.MapName, mp.MapID, m.CompoundName, m.Formula, m.FormDegen, m.MIM, m.CompoundID, m.MetacycUID, " 
												+ "mm.GlyphShape, mm.GlyphX, mm.GlyphY, mm.GlyphWidth, mm.GlyphHeight "
			+ "FROM Map mp, Metabolite m, MetaboliteOfMap mm "
			+ "WHERE mm.MapID = mp.MapID  "
			+ "AND mm.CompoundID = m.CompoundID "
			+ "ORDER BY mp.MapName";
			
	// Gets organism code from organism name
	final static String query2 = "SELECT OrganismID FROM Organism WHERE OrganismName = ? ";

	// creates an array of all ParamQuerys
	static ParamQuery pqList[] = 
			{ new ParamQuery(name0, query0),
			new ParamQuery(name1, query1), 
			new ParamQuery(name2, query2)
			};
			
	// find ParamQuery object by queryName and returns
	public static ParamQuery getParamQuery(String name) 
	{
		for (int i = 0; i < pqList.length; i++) 
		{
			if (pqList[i].getQueryName().equals(name)) 
			{
				return pqList[i];
			}
		}
		return null;
	}
			
	/* --- Constants for simple entity queries --- */
	   								 										
	static String allOrgNamesQuery = "SELECT DISTINCT OrganismName " +
										"FROM Organism " +
										"ORDER BY OrganismName ";
										
	/* --- Get Methods for simple entity queries  --- */
		
	//returns SQL to find all motif names	
	public static String getOrganismNamesQuery()
	{
		return allOrgNamesQuery;
	}

}
