/*
Metabolomics Web App 
Main class (originally entitled Metabolism)
DPL 02.09.2011
*/

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*;
import com.oreilly.servlet.multipart.*;

public class Pathos extends HttpServlet 
{	
	//------------------- start of repeat of MetabDB stuff -------------------//
	String username, password;
	PrintWriter out;
	Connection conn;
	HttpSession session; 	
	HttpServletRequest request;	
	
	// host info (obtained from init param)
	String host = null;
		boolean initFlag = false;	// for debugging
		
	final String LOCAL = "local";
	final String DOOLITTLE = "doolittle";
	final String MOTIF = "motif";
	final String MINNIE = "minnie";
	
	String hostURL = null;
	String hostServer = null;
	boolean proxy = false;
	boolean google = false;
	String proxyHost = "wwwcache.gla.ac.uk";	// for external cgi calls - not yet used
	String proxyPort = "8080";					// for external cgi calls - not yet used
	
	// String variables for parameters retrieved by request.getParameter() and corresponding names
	String menuChoiceParam; 		String menuChoice = "menuChoice";	
	String loadChoiceParam;			String loadChoice = "loadChoice";
	String analyseChoiceParam;		String analyseChoice = "analyseChoice";
	String analyseCompoundsParam;	String analyseCompounds = "analyseCompounds";	
	
	// String variables for safe coding of menu names and corresponding integers
	final int UPLOAD = 0;		String menUpload = "Upload File";
	final int ANALYSE = 1;		String menAnalyse = "Analyse";
	final int CPD_ANALYSE = 2;	// uses menAnalyse
	final int FEEDBACK = 2;		String menFeedback = "Feedback";
		
	// String array of values of menu buttons for DBMenu object
	String menuValues [] = {
							menUpload,
							menAnalyse,
							menFeedback
							};	
	
	// construct DBMenu to offload generation of left div with html menu code
	DBMenu dbMenu = new DBMenu(menuChoice, menuValues);
	
	// results variables as strings to allow html printing in any order
	StringBuffer resultsBuffer; 	// holds results of query	
	
	// Array of all Maps in organism(s)
	public Map[] mapSet;					// Array					
	private final int MAP_LENGTH = 5000;	// Capacity
	private int mapSize = 0; 				// Occupancy
	
	// safe coding of strings
	String allOrgs = "All Organisms";

	//------------------- File Format stuff -------------------//
	
	// boolean compoundInput = false;		// true indicates list of Compound IDs, else false indicates m/z input
	int format = -1;						// Type of input file, compound (Kegg or Metacyc) or +, -, or 0 m/z
	final int ERROR = -1;
	final int CPD = 0;
	final int POS = 1;
	final int NEG = 2;
	final int NEUT = 3;
	String modeDescriptor = null;		// Textual description of format for m/z
	
	// for compound input
	boolean keggInput = false;			// KeggIDs - only has a meaning if compoundInput is true
	boolean metaInput = false;			// MetacycUIDs - only has a meaning if compoundInput is true
	
	// For Peak array
	private Peak [] peakList;
	private int peakListLength = 10000;
	private int peakListSize = 0;
	
	// ...or Array of CompoundData objects
	private CompoundData [] cpdList;
	private int cpdListLength = 1000;
	private int cpdListSize = 0;
	
	// AdductList 
	private AdductList addList = new AdductList();	// must instantiate adduct list here
	private boolean prevAdducts = false;			// indicates initial choice - must reset on file load
	
	String fileName = null;			// Uploaded filename
	
	String[] conditions = null;		// array of conditions
	int numConditions = 0;			// Number of different expt condns
	int baseCondition = 0;			// Condition number (counting from 1) for baseline (0 = no cf)
	int exptCondition = 0;			// Condition number (counting from 1) for expt (0 = no cf.)
	
	// Double array of values for colour coding cutoff menu 
	double upValues [] = { 
							10.0, 9.0, 8.0, 7.0, 6.0, 5.0,
							4.0, 3.0, 2.0, 1.7, 1.5, 1.2
							};
							
	double downValues [] = { 
							0.8, 0.7, 0.6, 0.5,
							0.4, 0.3, 0.2, 0.1
							};	
	
	// colour coding cutoffs
	double up3, up2, up1, down1, down2, down3;
	// initial values - set on file upload
	final double DEF_UP3 = 10;
	final double DEF_UP2 = 5;
	final double DEF_UP1 = 2;
	final double DEF_DOWN1 = 0.5;
	final double DEF_DOWN2 = 0.2;
	final double DEF_DOWN3 = 0.1;
	
	ExptSettings settings;
	
	// get init-param specifying host - Java/XML book - DPL
	public void init(ServletConfig config) throws ServletException
    {
        super.init(config);
        host = config.getInitParameter("host");
        if (host == null)
        {
            host = LOCAL;
        }
        else
        {
        	initFlag = true;	// for debugging
        }
        
        if (host.equals(LOCAL))
        {
        	hostURL ="http://127.0.0.1";
   			hostServer = "127.0.0.1";
        	proxy = false;
        }
        else if (host.equals(DOOLITTLE))
        {
			hostURL = "http://doolittle.ibls.gla.ac.uk";
   			hostServer = "doolittle.ibls.gla.ac.uk";
			proxy = true;        
        }
        else if (host.equals(MOTIF))
        {
			hostURL = "http://motif.gla.ac.uk";
   			hostServer = "motif.gla.ac.uk";
			proxy = true;
			google = true;       
        }        
        else if (host.equals(MINNIE))
        {
			hostURL = "http://pc53-78.ibls.gla.ac.uk";
			proxy = true;        
        }
        else		// assume localhost
        {
        	hostURL ="http://127.0.0.1";
   			hostServer = "127.0.0.1";
        	proxy = false;
        }
    }
	
	public void doPost(HttpServletRequest request, HttpServletResponse response)
	                            			throws IOException, ServletException 
	{
		setUniProxy(proxyHost, proxyPort, proxy);	// set web proxies for external calls if from Uni-based server
			
		this.request = request;
		session = request.getSession(true);	// gets current session or create new one
		conn = (Connection) session.getAttribute("connection"); // get connection from session tracking details
	
		response.setContentType("text/html");
		out = response.getWriter();
		
		// Define the 'choice' param that will determine page generated			
		menuChoiceParam = request.getParameter(menuChoice);
		loadChoiceParam = request.getParameter(loadChoice);
		analyseChoiceParam = request.getParameter(analyseChoice);
		analyseCompoundsParam = request.getParameter(analyseCompounds);
		
		/*-- Start dynamically-generated html with hard-coded header --*/ 		
			writePageHead();   
		
		/*--Dynamically generate specifics of html page depending on params--*/
		
		// NEVER PRESSED ANYTHING: INITIALISE UPLOAD
		if (menuChoiceParam == null && loadChoiceParam == null && 
									analyseChoiceParam == null && analyseCompoundsParam == null)				
		{	      			
			username = "gordon";		// automatic guest login...
			password = "brown"; 
			connectMySQL();				// ...and connection..
			
			// ...and generates Initial page menu div -- this time for ANALYSE PAGE
			dbMenu.writeMenu(menuValues[ANALYSE], out, hostURL);
			writeMainDivStart();	

					/*-- Try to read file and put contents into array  --*/ 
			boolean parsed = true;
			peakList = new Peak[peakListLength];
			peakListSize = 0;				// resets in case of second upload	
				// Alternative for list of Compound IDs
			cpdList = new CompoundData[cpdListLength];
			cpdListSize = 0;

			try 
			{
				MultipartParser parser = new MultipartParser(request, 5 * 1024 * 1024, true, true);			
				
				FilePart part = (FilePart) parser.readNextPart();
				
				if(part.getFileName() != null)
				{
					fileName = part.getFileName();
				}				
				
				InputStream stream = part.getInputStream();
				InputStreamReader iReader = new InputStreamReader(stream);
				BufferedReader bReader = new BufferedReader(iReader);
				
				// parse first line to get conditions and input type
				String line1 = bReader.readLine();				
				StringTokenizer stk = new StringTokenizer(line1, "\t");
				String tok = stk.nextToken();
				
				// first char indicates input type
				if(tok.charAt(0) == 'C' || tok.charAt(0) == 'c')		// Kegg - C because Kegg numbers start with C
				{
					format = CPD;
					keggInput = true;
					metaInput = false;
					
					numConditions = Integer.parseInt(tok.substring(1, tok.length()));
					conditions = new String[numConditions];						
				}
				else if(tok.charAt(0) == 'M' || tok.charAt(0) == 'm')	// MetaCyc
				{
					format = CPD;
					keggInput = false;
					metaInput = true;
					
					numConditions = Integer.parseInt(tok.substring(1, tok.length()));
					conditions = new String[numConditions];	
				}
				else if(tok.charAt(0) == 'U' || tok.charAt(0) == 'u')	// m/z, Neutral
				{
					format = NEUT;
					modeDescriptor = "neutral";

					keggInput = false;	//  (variable should never be tested)
					metaInput = false;	//  (variable should never be tested)
					
					numConditions = Integer.parseInt(tok.substring(1, tok.length()));
					conditions = new String[numConditions];						
				}
				else if(tok.charAt(0) == 'P' || tok.charAt(0) == 'p')	// m/z, Positive
				{
					format = POS;
					modeDescriptor = "positive";
					keggInput = false;	// (variable should never be tested)
					metaInput = false;	// (variable should never be tested)
									
					numConditions = Integer.parseInt(tok.substring(1, tok.length()));
					conditions = new String[numConditions];						
				}
				else if(tok.charAt(0) == 'N' || tok.charAt(0) == 'n')		// m/z, Negative
				{
					format = NEG;
					modeDescriptor = "negative";
					keggInput = false;	// (variable should never be tested)
					metaInput = false;	// (variable should never be tested)
				
					numConditions = Integer.parseInt(tok.substring(1, tok.length()));
					conditions = new String[numConditions];						
				}
				else		// writes error message and sets flag so can prevent writing form
				{
					format = ERROR;
					out.println("<span style=\"color:red; font-weight:bold;\">File lacks initial identifying character:" + 
					" P, N, U, C or M, according to data type and mode. Please check Instructions.</span>");
				}
				
				prevAdducts = false;		// reset so adducts menu (if any) will load core set
				
				// If conditions, determine which is 'base' and which 'experimental' (if stated)
				// Format is base:experimental
				if(numConditions > 0)
				{
					String details = stk.nextToken();
					if(details.indexOf(":") != -1)
					{
						try
						{
							baseCondition = Integer.parseInt(details.substring(0, details.indexOf(":")));
							exptCondition = Integer.parseInt(details.substring(details.indexOf(":") + 1, details.length()));				
						}
						catch (NumberFormatException nfe)
						{
							out.println("Uploaded file appears to have an error in initial line! Please check.");
						}
					}
					// Set initial colour cutoffs on file upload so that they don't persist for servlet instance
					up3 = DEF_UP3;
					up2 = DEF_UP2;
					up1 = DEF_UP1;
					down1 = DEF_DOWN1;
					down2 = DEF_DOWN2;
					down3 = DEF_DOWN3;
				}
				else
				{
					baseCondition = 0;
					exptCondition = 0;
				}
				
				for(int i=0; i<numConditions; i++)
				{
					conditions[i] = stk.nextToken();
				}
				
				// now have to divide this up into two alternative streams
				if(format == CPD)	// either KeggID or MetacycUID input
				{
					// parse subsequent non-blank lines
					String line;
					while((line=bReader.readLine()) != null && !line.trim().equals(""))
					{
						try
						{
							StringTokenizer st = new StringTokenizer(line, "\t");								
							// first field is compoundID
							String compID = st.nextToken().toUpperCase();	
							CompoundData  cpd = new CompoundData(compID, numConditions);
							// expand array if necessary
							if(cpdListSize >= cpdListLength)
							{
								cpdListLength = 2 * cpdListLength;
								CompoundData[] newList = new CompoundData[cpdListLength];
								System.arraycopy (cpdList, 0, newList, 0, cpdList.length);
								cpdList = newList;
							}
							cpdList[cpdListSize] = cpd;
							cpdListSize++;
							// now add intensity values as integers
							for(int i=0; i<numConditions; i++)
							{
								Double valueObj = Double.valueOf(st.nextToken()); 
								double valueDouble = valueObj.doubleValue();
								int value = (int) valueDouble;
								cpd.addValue(value, i);
							}
							// now add error values as integers
							for(int j=0; j<numConditions; j++)
							{
								if(st.hasMoreTokens())
								{
									Double errorObj = Double.valueOf(st.nextToken()); 
									double errorDouble = errorObj.doubleValue();
									int error = (int) errorDouble;
									cpd.addError(error, j);
								}
								else
								{
									cpd.addError(0, j);
								}
							}
						}
						catch (NumberFormatException nfe)
						{
							out.println("Uploaded file appears to be in an incorrect format! Please check.");
							break;
						}
					}			
				}
				else				// M/Z data
				{
					// parse subsequent non-blank lines
					String line;
					while((line=bReader.readLine()) != null && !line.trim().equals(""))
					{
						try
						{
							StringTokenizer st = new StringTokenizer(line, "\t");								
							// first field is peak m/z
							Double massObj = Double.valueOf(st.nextToken()); 
							double massDouble = massObj.doubleValue();
							// construct peak object w/o intensities
							Peak  peak = new Peak(massDouble, numConditions);
							// expand array if necessary
							if(peakListSize >= peakListLength)
							{
								peakListLength = 2 * peakListLength;
								Peak[] newList = new Peak[peakListLength];
								System.arraycopy (peakList, 0, newList, 0, peakList.length);
								peakList = newList;
							}				
							peakList[peakListSize] = peak;
							peakListSize++;
							// now add intensity values as integers
							for(int i=0; i<numConditions; i++)
							{
								Double valueObj = Double.valueOf(st.nextToken()); 
								double valueDouble = valueObj.doubleValue();
								int value = (int) valueDouble;
								peak.addValue(value, i);
							}
							// now add error values as integers
							for(int j=0; j<numConditions; j++)
							{
								if(st.hasMoreTokens())
								{
									Double errorObj = Double.valueOf(st.nextToken()); 
									double errorDouble = errorObj.doubleValue();
									int error = (int) errorDouble;
									peak.addError(error, j);
								}	
								else
								{
									peak.addError(0, j);
								}	
							}
						}
						catch (NumberFormatException nfe)
						{
							out.println("Uploaded file appears to be in an incorrect format! Please check.");
							break;
						}
					}
				}
			}
			catch (IOException e)
			{ 
				out.println("Problem loading file!"); 
				parsed = false;
			}
			
			// Generate different Analysis form for the different situations
			if(format == CPD)
			{
				boolean nocompounds = false;			
				if(cpdListSize == 0)
				{
					parsed = false;
					nocompounds = true;
				}
				if(parsed)
				{
					writeAnalysisForm("");	// generates html for compound analysis
					writeInstructions(CPD_ANALYSE); 
				}
				else if (nocompounds)
				{
					out.println("Uploaded file does not contain compound IDs!");
				}
			}
			else if (format != ERROR)		// M/Z input
			{
				boolean nomasses = false;			
				if(peakListSize == 0)
				{
					parsed = false;
					nomasses = true;
				}			
				if(parsed)
				{
					writeAnalysisForm("", "2");	// generates html for analysis-specific stuff, no prev organism and 2ppm defaults
					writeInstructions(ANALYSE); 
				}
				else if (nomasses)
				{
					out.println("Uploaded file does not contain peak values!");
				}			
			}

			writeMainDivEnd();
		}
		
		// OTHER OPTIONS
		// Go to Upload File page
		 else if (menuChoiceParam != null && menuChoiceParam.equals(menUpload)) 
 		{	
 			dbMenu.writeMenu(menuValues[UPLOAD], out, hostURL);				
			writeUploadSection();			// generates html for upload-specific stuff
			nullifyRequestParams();
 		} 
		// Go to Analyse Data page (only occurs if gone to upload page and changed mind)
		 else if (menuChoiceParam != null && menuChoiceParam.equals(menAnalyse)) 
 		{	
 			dbMenu.writeMenu(menuValues[ANALYSE], out, hostURL);	
 						
			writeMainDivStart();
			if(format == CPD)
			{
				writeAnalysisForm("");	// generates html for analyse-specific stuff - 2ppm and +ve defaults
				writeInstructions(CPD_ANALYSE); 			
			}
			else
			{
				writeAnalysisForm("", "2");	// generates html for analyse-specific stuff - 2ppm and +ve defaults
				writeInstructions(ANALYSE); 
			}
			writeMainDivEnd();
			nullifyRequestParams();
 		} 
 		
 		 // Go to feedback page
		 else if (menuChoiceParam != null && menuChoiceParam.equals(menFeedback)) 
 		{	
 			dbMenu.writeMenu(menuValues[FEEDBACK], out, hostURL);	// generates Feedback Page menu	div	
			writeFeedbackPageSection();		// generates html for feedback form
			nullifyRequestParams();
 		}
  		
 		 				//	pressed button to execute query	//
 		 				
 		else if (analyseChoiceParam != null && analyseChoiceParam.equals("Run"))
 		{
 			dbMenu.writeMenu(menuValues[ANALYSE], out, hostURL);	// rewrite analyse page		
 			executeAnalysisQuery();		
 			nullifyRequestParams();
 		}
 		
 		else if (analyseCompoundsParam != null && analyseCompoundsParam.equals("Run"))
 		{
 			dbMenu.writeMenu(menuValues[ANALYSE], out, hostURL);	// rewrite analyse page
 			executeAnalysisQuery();		
 			nullifyRequestParams(); 			
 		}

 		// Error			 
		else
		{	
			dbMenu.writeMenu(menuValues[ANALYSE], out, hostURL);	// write home page
			writeMainDivStart();
			out.println("Choice not recognized. Perhaps it awaits implimentation.");
			writeMainDivEnd();
		}	
				
  		/*--Finish dynamically-generated html--*/ 
  			writePageFoot();  		  			  				
	}
	
	 	// resets all request parameters to null
 	private void nullifyRequestParams()
 	{
 		menuChoiceParam = null; 
 		loadChoiceParam = null;
 		analyseChoiceParam = null; 
 		analyseCompoundsParam = null;
 	}	

    //	doGet method - redirect to doPost code
   	public void doGet(HttpServletRequest request, HttpServletResponse response)
                      throws IOException, ServletException	
 	{
		doPost(request,response);
	}	

	//  Load MySQL driver, Get url, and connection to Mysql	
	void connectMySQL()
	{ 
		conn = null;
		try 
    	{
    		Class.forName("org.gjt.mm.mysql.Driver");	
    		conn = DriverManager.getConnection("jdbc:mysql://localhost/Metabolism",
    					username, password); 
			session.setAttribute("connection", conn);						
		}
		catch(SQLException e)
		{
			out.println("Trying to connect to MySQL" + e);
		}
		catch(Exception e)
		{
			out.println("Exception connecting to MySQL");
			e.printStackTrace(out);
		}
	}
	
	  		/*--Methods for writing hard-coded portions of html--*/
	
	//	Write HTML page head (and start of body)
	void writePageHead()
	{
		out.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
		out.println("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">");
		// html head
		out.println("<head>");		
		out.println("<title>Pathos: Metabolomics Web Facility</title>");
		out.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\" />");
		if(host.equals(DOOLITTLE))
		{
			out.println("<script src=\"" + hostURL + "/scripts/metab.js\" type=\"text/javascript\"></script>");			// Apache script dir
			out.println("<script src=\"" + hostURL + "/scripts/kAjax.js\" type=\"text/javascript\"></script>");			// ditto
			out.println("<script src=\"" + hostURL + "/scripts/dragresize.js\" type=\"text/javascript\"></script>");	// ditto
			out.println("<script src=\"" + hostURL + "/scripts/init.js\" type=\"text/javascript\"></script>");	// ditto
			out.println("<link rel=\"Stylesheet\" href=\"" + hostURL + "/scripts/metab.css\" type=\"text/css\" media=\"screen\" />"); 	// ditto
			out.println("<link rel=\"Stylesheet\" href=\"" + hostURL + "/scripts/metabPrint.css\" type=\"text/css\" media=\"print\" />"); 	// ditto			
			out.println("<link rel=\"Stylesheet\" href=\"" + hostURL + "/scripts/dragresize.css\" type=\"text/css\" />"); // ditto
		}
		else
		{
			out.println("<script src=\"" + "scripts/metab.js\" type=\"text/javascript\"></script>");		// servlet script dir metab.js
			out.println("<script src=\"" + "scripts/kAjax.js\" type=\"text/javascript\"></script>");		// servlet script dir kAjax.js
			out.println("<script src=\"" + "scripts/dragresize.js\" type=\"text/javascript\"></script>");	// servlet script dir dragresize.js
			out.println("<script src=\"" + "scripts/init.js\" type=\"text/javascript\"></script>");			// servlet script dir init.js
			out.println("<link rel=\"Stylesheet\" href=\"" + "scripts/metab.css\" type=\"text/css\" media=\"screen\" />"); 	// ditto metab.css
			out.println("<link rel=\"Stylesheet\" href=\"" + "scripts/metabPrint.css\" type=\"text/css\" media=\"print\" />"); 	// ditto metabPrint.css
			out.println("<link rel=\"Stylesheet\" href=\"" + "scripts/dragresize.css\" type=\"text/css\" />"); // ditto dragresize.css
			out.println("<link rel=\"icon\" href=\"http://motif.gla.ac.uk/metab.ico\" type=\"image/x-icon\" />");			// web icon
			out.println("<link rel=\"shortcut icon\" href=\"http://motif.gla.ac.uk/metab.ico\" type=\"image/x-icon\" />");	// web icon
			out.println("<link href=\"http://motif.gla.ac.uk/pathos.xml\" rel=\"alternate\" type=\"application/rss+xml\" title=\"rss\" />");	// RSS feed link
		}
    	out.println("</head>");	
		// body-start, title and hori bar	
		out.println("<body onload=\"stripeTable('stripey', '#ffffff', '#edfef3'); createLink('visible','hideme','expand');" + 
			"createLink('visibleA','hidemeA','expandA'); createLink('visibleB','hidemeB','expandB'); createLink('visibleC','hidemeC','expandC');\">");
		out.println("<div id=\"wrapper\">");
		out.println("<div class=\"masthead\">");
		out.println("<h1>Pathos</h1>");
		out.println("<h2>From Mass Spec Peak to Metabolic Map</h2>");
		out.println("</div>");
	}	

	void writePageFoot()
	{	
		StringBuffer footy = new StringBuffer("");
		footy.append("<div class=\"footer\">\n");		
				footy.append("<div style=\"width:400px; float:right; text-align:right;\">");	
		footy.append("<a href=\"mailto:david.leader@gla.ac.uk\">David P. Leader (University of Glasgow)</a>");
		footy.append("</div>");	
		footy.append("<div>A data-analysis tool from the Scottish Metabolomics Facility</div>");
		footy.append("</div>\n");	// end of footer div
		footy.append("</div>\n");	// end of wrapper div	
		
		// Google analytics code - based on previous, not actually as currently supplied by Google
		if(google)
		{
			footy.append("<script type=\"text/javascript\"> var gaJsHost = ((\"https:\" == document.location.protocol) ? \"https://ssl.\" : \"http://www.\");");
			footy.append("document.write(unescape(\"%3Cscript src='\" + gaJsHost + \"google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E\")); </script>");
			footy.append("<script type=\"text/javascript\"> var pageTracker = _gat._getTracker(\"UA-3315042-13\");");
			footy.append("pageTracker._initData();");
			footy.append("pageTracker._trackPageview(); </script>");
		}
					
		footy.append("</body>\n");
		footy.append("</html>\n");
		
     	out.println(footy.toString());     	
 	}

 		  		//*-- DYNAMIC PORTIONS OF HTML & VARIOUS QUERIES --*//
 		  					/*-- UPLOAD QUERY --*/
 	
 	// single version of upload page without repeat upload section - possibly modify later	  		
	void writeUploadSection()
	{
		writeMainDivStart();
		out.println(readHTML("htmlText/homepage.txt"));	// external html block for Home Page
		writeUploadForm();
		writeInstructions(UPLOAD);
		writeMainDivEnd();	
	}
	
	void writeUploadForm()
	{		
		out.println("<form id=\"queryEntry\" action=\"/Pathos/pathos.html\" enctype=\"multipart/form-data\" method=\"post\" onSubmit=\"return validateFile('uFile');\">");
		out.println("<div class=\"action\">");
		out.println("<input type=\"file\" name=\"file\" id=\"uFile\" />");
		
		// submit button and end of form
		out.println("<input type=\"submit\" value=\"Upload\" name=\"" + loadChoice + 
																"\" class=\"shiftRight\" />");
		out.println("</div>");
		out.println("</form>");		
	}
	
	 		  				/*-- ANALYSE UPLOAD QUERY --*/
	 		  				
	// This is the main function - analyses uploaded masses for metabolites present in selected organism
	
	// For compoundInput
	void writeAnalysisForm(String prevOrg)
	{
		writeAnalysisForm(prevOrg, null);
	}
	
	// writes form with previous choices (if any) as defaults
	void writeAnalysisForm(String prevOrg, String prevPPM)
	{
		out.println("<form id=\"queryEntry\" action=\"\" method=\"post\" onsubmit=\"return(identityCheck());\">");
		out.println("<div class=\"action\">");	
		
		// populate drop-down list of organisms for form
		out.println("<div style=\"float:left;\">");		// need to float for adducts options
		out.println("Organism:");
		try
		{
			writeOrganismOptions(prevOrg);
		}
		catch (SQLException sqe)
		{
			out.println(sqe);
		}
		out.println("</div>");
		
		// populate drop-down +ve/-ve/Neutral mode options for form 
		if(format != CPD)
		{
			if(format != NEUT)	// only write adducts menu for +ve and -ve
			{
				// float adducts label
				out.println("<div style=\"float:left;\">");
				out.println("&nbsp;&nbsp;Adduct(s):&nbsp;");
				out.println("</div>");
				
				// populate adducts select options
				out.println("<div style=\"float:left;\">");
				out.println("<select name=\"adducts\" size=\"7\" multiple=\"multiple\" style=\"padding:2px;\">");
				
				for (int i=0; i<addList.getSize(); i++)
				{
					// get details for ea adduct so can write it as necessary
					Adduct add = addList.getAdduct(i);
					int id = add.getID();	// ID
					String longDes = add.getLongDescriptor();	// long descriptor
					boolean pos = add.isPositive();		// +ve mode
					// Add positive or negative adducts, selected or not, as appropriate
					if(pos == true && format == POS || pos == false && format == NEG)
					{
						// core adduct initially or previously selected adduct
						if( (prevAdducts == false && add.isRestricted()) || (prevAdducts == true && add.isSelected()) )	
						{
							out.println("<option selected=\"selected\" value=\"" + id +"\">" + longDes + "</option>");
						}
						else	// not core or previously selected adduct
						{
							out.println("<option value=\"" + id +"\">" + longDes + "</option>");	
						}
					}
				}
				out.println("</select>");
				out.println("</div>");		
			}
				
			// populate drop-down list of ppms irrespective of m/z mode
			out.println("<div style=\"float:left;\">");
			out.println("&nbsp;&plusmn;ppm:");
			out.println("<select name=\"ppm\">");	
			for (int i=1; i<11; i++)
			{
				String iString = Integer.toString(i);
				if(iString.equals(prevPPM))		
				{
					out.println("<option selected=\"selected\" value=\"" + i + "\">" + i + "</option>");
				}	
				else
				{
					out.println("<option value=\"" + i + "\">" + i + "</option>");			
				}	
			}
			out.println("</select></div>");
		}		
		
		// submit button and end of form for both compound and m/z
		out.println("<div style=\"float:right;\">");
		out.println("<input type=\"submit\" value=\"Run\" name=\"" 
						+ analyseChoice + "\" class=\"shiftRight\" />");
		out.println("</div>");				
		out.println("<div class=\"clearer\"></div>");	
						
		out.println("</div>");
		
		// extra section for experimental comparison indication/choice
		if(numConditions > 0)
		{
			out.println("<div class=\"action\" style=\"margin-top: 5px;\">");
			out.println("<div>");
			out.println("Base Condition:");
			out.println("<select name=\"base\" id=\"bs\">");
			for(int i=0; i<numConditions; i++)
			{
				if( i+1 == baseCondition)	// conditions numbered in file from 1 for non-tech users
				{
					out.println("<option selected=\"selected\" value=\"" + (i+1) + "\">" + conditions[i] + "</option>");
				}
				else
				{
					out.println("<option value=\"" + (i+1) + "\">" + conditions[i] + "</option>");
				}
			}
			out.println("</select>");

			out.println("&nbsp;Experimental Condition:");
			out.println("<select name=\"expt\" id=\"ex\">");
			for(int i=0; i<numConditions; i++)
			{
				if( i+1 == exptCondition)	// conditions numbered in file from 1 for non-tech users
				{
					out.println("<option selected=\"selected\" value=\"" + (i+1) + "\">" + conditions[i] + "</option>");
				}
				else
				{
					out.println("<option value=\"" + (i+1) + "\">" + conditions[i] + "</option>");
				}
			}
			out.println("</select>");
			out.println("</div>");
			
			// colour cut-off stuff
			out.println("<div style=\"padding-top: 10px;\">");
			out.println("Cut-offs for colour-flagging:"); 
			
			out.println("<select name=\"u3\" id=\"up3\">");
			for(int i=0; i<upValues.length-2; i++)
			{
				if(upValues[i] == up3)
				{
					out.println("<option selected=\"selected\" value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");
				}
			}			
			out.println("</select>");
			

			out.println("<select name=\"u2\" id=\"up2\">");
			for(int i=1; i<upValues.length-1; i++)
			{
				if(upValues[i] == up2)
				{
					out.println("<option selected=\"selected\" value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");
				}
			}
			out.println("</select>");

			out.println("<select name=\"u1\" id=\"up1\">");
			for(int i=2; i<upValues.length; i++)
			{
				if(upValues[i] == up1)
				{
					out.println("<option selected=\"selected\" value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + upValues[i] + "\">" + upValues[i] + "&times;</option>");
				}
			}			
			out.println("</select>");

			out.println("<select name=\"d1\" id=\"down1\">");
			for(int i=0; i<downValues.length-2; i++)
			{
				if(downValues[i] == down1)
				{
					out.println("<option selected=\"selected\" value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");
				}
			}			
			out.println("</select>");


			out.println("<select name=\"d2\" id=\"down2\">");
			for(int i=1; i<downValues.length-1; i++)
			{
				if(downValues[i] == down2)
				{
					out.println("<option selected=\"selected\" value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");
				}
			}
			out.println("</select>");

			out.println("<select name=\"d3\" id=\"down3\">");
			for(int i=2; i<downValues.length; i++)
			{
				if(downValues[i] == down3)
				{
					out.println("<option selected=\"selected\" value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");				
				}
				else
				{
					out.println("<option value=\"" + downValues[i] + "\">" + downValues[i] + "&times;</option>");
				}
			}
			out.println("</select>");

			out.println("</div>");
			out.println("</div>");	
		}				
		out.println("</form>");		
	}
		  
	// starts analysis		
	void executeAnalysisQuery()
	{
	 	writeMainDivStart();
	 
	 	// Instantiate map array
		mapSet = new Map[MAP_LENGTH];
		
		// get common parameters
		String organism = request.getParameter("organism");
		boolean all = organism.equals("All Organisms");		
		
		// get m/z mode parameters
		String[] adductIDs = null;
		String ppmString = null;
		int ppm = 0;				
		if(format != CPD)
		{
			adductIDs = request.getParameterValues("adducts");
			ppmString = request.getParameter("ppm");
			ppm = Integer.parseInt(ppmString);
			if(format == POS || format == NEG)
			{
				adductIDs = request.getParameterValues("adducts");	// creates a full array of length adducts.length
				prevAdducts = true;
			}
		}
		
		// organism code 
		String orgCode = "map";				// default if 'all' true
		if(!all)
		{
			try
			{
				orgCode = getOrgCode(organism);
			}
			catch (SQLException sqle)
			{
				out.println(sqle.toString());
			}			
		}
		
		// get parameters for base and expt conditions etc (if any)
		if(numConditions > 0)
		{
			String baseString = request.getParameter("base");
			String exptString = request.getParameter("expt");
			
			String up3String = request.getParameter("u3");
			String up2String = request.getParameter("u2");
			String up1String = request.getParameter("u1");
			String down1String = request.getParameter("d1");
			String down2String = request.getParameter("d2");
			String down3String = request.getParameter("d3");
			 
			try
			{
				baseCondition = Integer.parseInt(baseString);
				exptCondition = Integer.parseInt(exptString);
				
				up3 = Double.parseDouble(up3String);
				up2 = Double.parseDouble(up2String);
				up1 = Double.parseDouble(up1String);
				down1 = Double.parseDouble(down1String);
				down2 = Double.parseDouble(down2String);
				down3 = Double.parseDouble(down3String);
				
				settings = new ExptSettings(baseCondition, exptCondition, up3, up2, up1, down1, down2, down3);
			}
			catch (NumberFormatException nfe){System.out.println(nfe.toString());}
		}

		// run query to populate mapSet
		try
		{
			initializeMaps(organism);
		}
		catch (SQLException sqle)
		{
			out.println(sqle.toString());
		}
		
		if(format == CPD)
		{
			// reset 'present' flag on all Compounds in list - probably not needed
			resetCompounds();		
			
			for (int i=0; i < cpdListSize ; i++)
			{
				CompoundData cpd =cpdList[i];
				String queryID = cpd.getCompoundID();	// remember this will be Kegg or Metacyc in diff circumstance
				
				// and look through set of maps
				for (int j=0; j < mapSize; j++)
				{
					Map map = mapSet[j];
					// for each map examine each metabolite
					for (int k=0; k < map.getNumMetabolites() ; k++)
					{
						Metabolite metab = map.getMetabolite(k);
						// and check if compoundID  or MetacycUiD corresponds
						String id = null;
						if(keggInput)
						{
							id = metab.getID();
						}
						else if(metaInput)
						{
							id = metab.getMetacycUID();
						}

						if(id !=null && queryID.equals(id))
						{
							// mark as present in map
							if(!metab.isPresent())		// this is prob not necessary
							{
								metab.setPresent(true);
								map.incrementPresent();
							}
							// mark as present in array of Peaks in either case -- prob not necessary
							cpd.setPresent(true);
							// add values and SE if appropriate
							if(numConditions > 0)
							{
								metab.add(cpd.getValues(), cpd.getErrors(), conditions);
							}
						}					
					}	
				}
			}	 	
		}
		else		// m/z mode
		{
			// Go through adduct list, deselect all, then flag those that have been selected
			addList.deselectAdducts();
			if(adductIDs != null)
			{
				for (int i=0; i<adductIDs.length; i++)
				{
					try
					{
						int id = Integer.parseInt(adductIDs[i]);
						Adduct add = addList.findByID(id);
						if(add != null)
						{
							add.setSelected(true);
						}
					}
					catch (NumberFormatException nfe)
					{System.out.println(nfe.toString());}
				}
			}
			
			// reset 'present' flag on all Peaks in list
			resetPeaks();
			sortPeaks();	
						
			// take each peak in the uploaded array 
			for (int i=0; i < peakListSize ; i++)
			{
				Peak peak = peakList[i];
				double queryMZ = peak.getMZ();
				// and look through set of maps
				for (int j=0; j < mapSize; j++)
				{
					Map map = mapSet[j];
					// for each map examine each metabolite
					for (int k=0; k < map.getNumMetabolites() ; k++)
					{
						Metabolite metab = map.getMetabolite(k);
						// and check if corresponds to mass within range (or ion species)
						double mass = metab.getMass();
						
						if(format == POS || format == NEG)	// only look at adducts for +ve and -ve mode
						{
							for (int l=0; l<addList.getSize(); l++)	// for each adduct
							{
								Adduct add = addList.getAdduct(l);
								if (add.isSelected() == true)	// check if selected
								{
									double mz = add.makeMZ(mass);
									if(queryMZ <= mz + (mz*ppm/1000000) && queryMZ >= mz - (mz*ppm/1000000) )
									{
										// mark as present in map
										if(!metab.isPresent())		// this is necessary because of multiple masses giving same hit
										{
											metab.setPresent(true);
											map.incrementPresent();
										}
										// mark as present in array of Peaks in either case, and append adduct
										peak.setPresent(true);
										peak.addFormula(metab, add); 
										// add values, SE, conditions and adduct
										if(numConditions > 1)
										{
											metab.add(peak.getValues(), peak.getErrors(), conditions, add.getDescriptor());
										}
										else
										{
											metab.addAdductSimple(add.getDescriptor());
										}
									}
								}
							}
						}
						else if (format == NEUT)	// neutral
						{
							if(queryMZ <= mass + (mass*ppm/1000000) && queryMZ >= mass - (mass*ppm/1000000) )
							{
								// mark as present in map
								if(!metab.isPresent())		// this is necessary because of multiple masses giving same hit
								{
									metab.setPresent(true);
									map.incrementPresent();
								}
								// mark as present in array of Peaks in either case
								peak.setPresent(true);
								peak.addFormula(metab.getFormula());
								// add values, SE and conditions
								if(numConditions > 1)
								{
									metab.add(peak.getValues(), peak.getErrors(), conditions);
								}
							}					
						}
					}	
				}
			}
		}
		
		// sort array of maps by population
		sortMaps();
		
		writeAnalysisForm(organism, ppmString);			// Repeat form *after* adducts have been set 
		
		formatAnalysisResults(organism, orgCode, ppmString);
		out.println(resultsBuffer.toString());
		writeMainDivEnd();
	}
	
	// formats results
	void formatAnalysisResults(String organism, String orgCode, String ppm)
	{
		int foundMapNum = 0;								// number of maps found containing 'hits'
		
		resultsBuffer = new StringBuffer("");				// initialize Overall SB for results - this has titles etc
		StringBuffer mapBuffer = new StringBuffer("");		// mapBuffer with stuff for individual pathway maps (will become part of resultsBuffer)
		StringBuffer noMapBuffer = new StringBuffer("");	// lists maps not represented - will become part of resultsBuffer
		if(format == CPD)
		{
			noMapBuffer.append("<div id=\"visible\"><strong>Maps containing No Compounds from Data Set: </strong></div>");			
		}
		else
		{
			noMapBuffer.append("<div id=\"visible\"><strong>Maps containing No potential Metabolites from Experiment: </strong></div>");	
		}
		noMapBuffer.append("<div id=\"hideme\" style=\"display: none;\">");

		for (int i = 0; i < mapSize; i++) 
		{
			if (mapSet[i] != null) 					
			{
				// Sort metabolites in ea Map alphabetically
				mapSet[i].sort();
				
				StringBuffer linkBuffer = new StringBuffer("<div class=\"generate\"> Generate map of ");	// SB for link to kAjax.js (will become part of mapBuffer)
				linkBuffer.append("<a href=\"#\" onclick=\"fetchMap(");
				
				StringBuffer stateBuffer = new StringBuffer("'");		// SB for 'state' var of kAjax.js (i.e. degree of change - will become part of linkBuffer)
				
				StringBuffer areaBuffer = new StringBuffer("<map name=\"keggmap" + i + "\" id=\"keggmap" + i + "\">");	// SB to hold imageMap details. (will become part of mapBuffer - not in linkBuffer)	
				
				Map map = mapSet[i];
				if (map.getNumPresent() == 0) 			// No metabolites in map - add map name to noMapBuffer
				{
					noMapBuffer.append("<div>");
					noMapBuffer.append(map.getMapName());
					noMapBuffer.append("</div>");
				} 
				else 									// Metabolites found - add link to map in mapBuffer
				{
					map.setDegeneracy();				// calculate degeneracies among those metabolites present
					// Add map (path) and organism info to linkBuffer for link to Kegg webservice map
					linkBuffer.append("'" + map.getMapIDnum() + "',");
					linkBuffer.append("'" + orgCode + "',");
					linkBuffer.append("'");
					
					mapBuffer.append("<div class=\"pathway\">");
					mapBuffer.append("<strong>");
					mapBuffer.append(map.getMapName());
					mapBuffer.append("</strong>");
					mapBuffer.append(": ");
					mapBuffer.append(map.getUniquePresent());
					if(map.getUniquePresent() > 1)
					{
						mapBuffer.append(" metabolites out of ");
					}
					else
					{
						mapBuffer.append(" metabolite out of ");					
					}
					mapBuffer.append(map.getUniqueMetabolites());
					mapBuffer.append(" <span class=\"bar\"><a href=\"javascript:reveal('toggle" + i + "');\">&or;</a></span>");	// hide/show
					mapBuffer.append("</div>");
					
					mapBuffer.append("<div class=\"hideTable\" id=\"toggle" + i + "\">");
					mapBuffer.append("<table cellspacing=\"0\" class=\"tabRes\">");		// start table of results
					
					foundMapNum++;		// increment count of maps with metabolites found
				}


				// Make array to hold found metabolite ids so can test for repeats (diff map locs) and hide one of them in the listing
				String[] foundIDs = new String[map.getNumMetabolites()];
				int numIDs = 0;
					
				for (int j = 0; j < map.getNumMetabolites(); j++) 
				{
					if (map.getMetabolite(j).isPresent()) // For ea Metabolite found, add to list with link
					{
						String cpdID = map.getMetabolite(j).getID();
						
						// check if already preset in array of found IDs
						boolean alreadyThere = false;
						for(int k=0; k<numIDs; k++)
						{
							if (cpdID.equals(foundIDs[k]))
							{
								alreadyThere = true;
							}
						}					

						foundIDs[numIDs] = cpdID; numIDs++;			// add ID to list
						String cpdName = map.getMetabolite(j).getName();
						String cpdFormula = map.getMetabolite(j).getFormula();
						int formDegen = map.getMetabolite(j).getFormDegen();
						int degenPresent = map.getMetabolite(j).getDegenPresent();
						// ...and for imagemap
						String glyphShape = map.getMetabolite(j).getGlyphShape();
						int glyphX = map.getMetabolite(j).getGlyphX();
						int glyphY = map.getMetabolite(j).getGlyphY();
						int glyphWidth = map.getMetabolite(j).getGlyphWidth();
						int glyphHeight = map.getMetabolite(j).getGlyphHeight();
						
						// mapBuffer
						
						if(!alreadyThere)
						{
							mapBuffer.append("<tr><td>");			// start table row for this compound in map buffer		
						}	
						else
						{
							mapBuffer.append("<tr style=\"display:none;\"><td>");	// but hide if repeat (*must* write for Kegg map)
						}	
												
						// links to cf servlet for both listing and imagemap
						if(numConditions > 0)
						{
							mapBuffer.append(" " + map.getMetabolite(j).getComparisonImage());
							areaBuffer.append(map.getMetabolite(j).getAreaSpec());
						}
						
						/*	if(format == CPD)	// simple link if for <<compoundInput>>
						{
							mapBuffer.append("<a href=\"javascript:openNewWindow('http://www.genome.jp/dbget-bin/www_bget?compound+" + 
							cpdID + "','Kegg',650,500)\"> ");
						}*/

						// mapBuffer: custom pop-up with degeneracy info
						mapBuffer.append(" <a href=\"javascript:openNewWindow('" + hostURL + "/cgi-bin/formula.cgi?"
										+ "id=" + cpdID
										+ "&amp;name=" + URLEncoder.encode(escapeQuote(cpdName)) 
										+ "&amp;formula=" + cpdFormula
										+ "&amp;degeneracy=" + formDegen
										+ "&amp;host=" + hostServer
										+ "','Kegg%20Metabolite%20" + cpdID + "',450,350)\">");						
										
						mapBuffer.append(cpdName);
						mapBuffer.append("</a>");
						mapBuffer.append("  ");
						
						//if(format != CPD)	// start of span round formula
						mapBuffer.append("<span onmouseover=\"hilite('" + cpdFormula + "');\" onmouseout=\"lolite('" + cpdFormula + "');\">");
						
						mapBuffer.append(cpdFormula);
						
						// if(format != CPD)	// end of span round formula
						mapBuffer.append("</span>");
						
						if(formDegen == 1 && format != CPD)
						{
							mapBuffer.append(" *");			// asterisk is No Degeneracy flag
						}
						else if(degenPresent == 1 && format != CPD)
						{
							mapBuffer.append(" &dagger;");	//  is flag for No Degeneracy among those present in map
						}
						else if(format != CPD)
						{
							mapBuffer.append(" [" + degenPresent + "]");						
						}
											
						mapBuffer.append("</td><td>");	
						if(format == POS || format == NEG)
						{
							mapBuffer.append(map.getMetabolite(j).getAdducts());		// list of adducts
						}
						mapBuffer.append("</td></tr>");
						
						linkBuffer.append(map.getMetabolite(j).getLongID());		//ok
						linkBuffer.append("%2C");	// comma
						stateBuffer.append(map.getMetabolite(j).getState());
						stateBuffer.append("%2C");
	
					} 
				}
				// finish off link buffer by removing last comma and finishing the link 
				linkBuffer.delete(linkBuffer.length()-3, linkBuffer.length());			// remove comma (last three chars)
				linkBuffer.append("',");				// add ', to finish
				
				if(stateBuffer.length() >= 3)			// previously not necessary because of prologue
				{
					stateBuffer.delete(stateBuffer.length()-3, stateBuffer.length());		// do. for stateBuffer
				}
				stateBuffer.append("',");				// add ', to finish
				
				linkBuffer.append(stateBuffer.toString());								// append stateBuffer to linkBuffer
				
				linkBuffer.append("'mapWin" + i + "',");
				linkBuffer.append("'mapCont" + i + "',");		
				linkBuffer.append("'" + i + "'");				// no comma for last
				linkBuffer.append(");\"><strong>" + map.getMapName() + "</strong></a> highlighting potential metabolites.</div>");
				
				linkBuffer.append("<div class=\"drsElement\" id=\"mapWin" + i + "\" style=\"left:50px;top:100px;display:none;\">");
				linkBuffer.append("<div class=\"drsMoveHandle\">");		
				linkBuffer.append("<div class=\"closebox\"><a href=\"#\" onclick=\"closeMap(); return(false);\">&nbsp;&times;&nbsp;</a></div>");
				linkBuffer.append("</div><div class=\"mapContent\" id=\"mapCont" + i + "\">");
				linkBuffer.append("<p><img src=\"scripts/ajax-loader.gif\" alt=\"\" /></p></div>");			
				linkBuffer.append("<p class=\"resize\"><img class=\"thumbDragImg\" src=\"scripts/corner.gif\" alt=\"\" /></p></div>");
				
				// finish off area Buffer
				areaBuffer.append("</map>");
				
				// add to mapBuffer only if map has metabolites
				if (map.getNumPresent() != 0) 
				{
					mapBuffer.append("</table>");							// finish table of results for that map
					mapBuffer.append("</div>");								// finish table div
					mapBuffer.append(linkBuffer.toString());				// add linkBuffer to mapBuffer
					mapBuffer.append(areaBuffer.toString());				// add areaBuffer to mapBuffer
				}
			}
		}
		noMapBuffer.append("</div>");	
		
		// masses not found 
		int absentCount = 0;									// count of masses/compounds not found
		StringBuffer notFoundBuffer = new StringBuffer("");		// Holds masses/compounds not found
		StringBuffer foundBuffer = new StringBuffer("");		// Holds formulae of found compounds
		
	
		if(format == CPD)
		{
			for(int i=0; i< cpdListSize; i++)
			{
				CompoundData cpd = cpdList[i];
				if(!cpd.isPresent())
				{
					notFoundBuffer.append("<div>" + cpd.getCompoundID() + "</div>");
					absentCount++;
				}
			}
			int presentCount = cpdListSize - absentCount;
			
			resultsBuffer.append("<p class=\"resultstitle\">File: &lsquo;" + fileName + "&rsquo;<br />");

			resultsBuffer.append("Kegg Maps for " + organism + " with compounds from input list</p>");
						
			resultsBuffer.append("<div id=\"visibleA\"><strong>KEY</strong></div>");
			if(foundMapNum>0)
			{		
				resultsBuffer.append("<div id=\"visibleR\">All maps <span class=\"bar\"><a href=\"javascript:showAll('" + foundMapNum + "',true);\">&or;</a></span></div>");
			}
			resultsBuffer.append("<p class=\"clearer\"></p>");
			
			resultsBuffer.append("<div id=\"hidemeA\" style=\"display: none;\">");		
		}
		else
		{			
			for(int i=0; i< peakListSize; i++)
			{
				Peak peak = peakList[i];
				if(!peak.isPresent())
				{
					notFoundBuffer.append("<div>" + peak.getMZ() + "</div>");
					absentCount++;
				}
			}
			int presentCount = peakListSize - absentCount;
			
			resultsBuffer.append("<p class=\"resultstitle\">File: &lsquo;" + fileName + "&rsquo;<br />");

			resultsBuffer.append("Potential Metabolites found for " + organism + " &mdash; " + presentCount + " of " + peakListSize + " peaks<br />");
			if(format == NEUT)
			{		
				resultsBuffer.append("Mode: " + modeDescriptor + ", Tolerance: &plusmn;" + ppm + " ppm</p>"); 
			}
			else
			{
				int selCount = addList.getNumSelected();
				if(selCount > 1)
				{
					resultsBuffer.append("Mode: " + modeDescriptor + ", " + selCount + " adducts selected, Tolerance: &plusmn;" + ppm + " ppm</p>");
				}
				else if (selCount == 1)
				{
					resultsBuffer.append("Mode: " + modeDescriptor + ", " + selCount + " adduct selected, Tolerance: &plusmn;" + ppm + " ppm</p>");				
				}
				else
				{
					resultsBuffer.append("Mode: " + modeDescriptor + ", <span style=\"color:red;\">No adducts selected!</span> Tolerance: &plusmn;" + ppm + " ppm</p>");				
				}
			}
			
			resultsBuffer.append("<div id=\"visibleA\"><strong>KEY</strong></div>");
			if(foundMapNum>0)
			{		
				resultsBuffer.append("<div id=\"visibleR\">All maps <span class=\"bar\"><a href=\"javascript:showAll('" + foundMapNum + "',true);\">&or;</a></span></div>");
			}
			resultsBuffer.append("<p class=\"clearer\"></p>");
			
			resultsBuffer.append("<div id=\"hidemeA\" style=\"display: none;\">");	
			resultsBuffer.append("<p>* : An asterisk indicates that a formula is unique to that metabolite in all Kegg maps.<br />" +
								"&dagger; : A dagger indicates that a formula is unique to that metabolite in the current Kegg map.<br />" +
								"In other cases the number of metabolites with the same formula in the current map is shown in square brackets, e.g. [3], and these may be highlighted by mousing over the formula.<br />" +					
								"Clicking on the name of a metabolite invokes a pop-up displaying its structure and listing all alternatives for the corresponding formula.</p>");	
		}
		
		resultsBuffer.append("<p><span class=\"bar\">&or;</span> : View metabolites found for a particular map (or all maps) &mdash; toggles list on and off.</p>");		
		if(numConditions > 0)
		{	
			resultsBuffer.append("<p><span class=\"bar\">G</span> : Link to a graph of results for a particular metabolite. Change indication scale &mdash; [increase <span class=\"bar3p\">G</span>" +  
				"<span class=\"bar2p\">G</span><span class=\"bar1p\">G</span><span class=\"bar0\">G</span><span class=\"bar1n\">G</span><span class=\"bar2n\">G</span><span class=\"bar3n\">G</span> decrease]</p>");		
		}	
		resultsBuffer.append("</div>");		// end of hide/show div
					
			
		resultsBuffer.append(mapBuffer.toString());
		resultsBuffer.append(noMapBuffer.toString());
		
		if(format != CPD)		// Non-identified peaks only for peak input
		{
			resultsBuffer.append("<div id=\"visibleB\"><strong>Experimental Peaks not corresponding to any Metabolite of a Kegg Map (" + absentCount + "): </strong></div>");
			resultsBuffer.append("<div id=\"hidemeB\" style=\"display: none;\">");
			resultsBuffer.append(notFoundBuffer.toString());	
			resultsBuffer.append("</div>");	
			
			// List found formulae
			for(int i=0; i< peakListSize; i++)
			{
				Peak peak = peakList[i];
				if(peak.isPresent())
				{
					foundBuffer.append("<div>" + peak.getMZ() + " : ");
					if(format == NEUT)
					{
						for(int j=0; j<peak.getFormulaeSize(); j++)
						{ 
							foundBuffer.append(peak.getFormula(j) + " ");
						}
					}
					else
					{
						for(int j=0; j<peak.getMetaddSize(); j++)
						{ 
							foundBuffer.append(peak.getMetAdd(j).getMetabolite().getFormula() + 
								"(" + peak.getMetAdd(j).getAdduct().getDescriptor() + ") " );
						}					
					}
					foundBuffer.append("</div>");
				}
			}
			resultsBuffer.append("<div id=\"visibleC\"><strong>Summary of Mass/Formula correlations for identified Peaks (" 
																				+ (peakListSize - absentCount) + "): </strong></div>");
			resultsBuffer.append("<div id=\"hidemeC\" style=\"display: none;\">");
			resultsBuffer.append(foundBuffer.toString());	
			resultsBuffer.append("</div>");				
		}			
	} 


  		  		/*-- METHODS FOR WRITING BOILERPLATE HTML --*/   
	
	void writeMainDivStart()
	{
		out.println("<div class=\"main\" id=\"content\">");	
	}
	
	void writeMainDivEnd()
	{
		out.println("<p class=\"clearer\"></p>");
		out.println("</div>");	
	}
	
	void writeResultsDivEnd()
	{
		out.println("</div>");	
	}
		
	 void writeHomepageSection() 
	{
		writeMainDivStart();
		out.println(readHTML("htmlText/homepage.txt"));	// external html block for Home Page
		writeMainDivEnd();	
	}

	void writeFeedbackPageSection() 
	{
		writeMainDivStart();
		out.println(readHTML("htmlText/feedbackForm.txt"));	// external html block for Feedback form
		writeMainDivEnd();	
	}
	
	// writes an instruction string for different queries
	void writeInstructions(int query)
	{
		if(query == UPLOAD)
		{
			out.println("<div class=\"descript\">");
			out.println("Select a suitable data file and then click &lsquo;Upload&rsquo;.");
			out.println("</div>");
		}
		else if (query == ANALYSE)
		{
			out.println("<div class=\"descript\">");
			if(peakListSize > 0 && fileName != null)
			{			
				out.println("Analyse uploaded list of " + peakListSize +" peaks from file &lsquo;" + fileName + "&rsquo;" +
				" in " + modeDescriptor + " mode.");
			}
			else
			{
				out.println("<span style=\"color:red;\">No peaks to analyse! Please check that you uploaded a valid file.</span>");			
			}
			out.println("</div>");		
		}
		else if (query == CPD_ANALYSE)
		{
			out.println("<div class=\"descript\">");
			if(cpdListSize > 0 && fileName != null)
			{
				out.println("Analyse uploaded list of " + cpdListSize +" compounds from file &lsquo;" + fileName + "&rsquo;.");
			}
			else
			{
				out.println("<span style=\"color:red;\">No compounds to analyse! Please check that you uploaded a valid file.</span>");			
			}
			out.println("</div>");	
		}
	}

  		  		/*-- UTILITY & OTHER METHODS --*/ 
  		  		
 	// method to generate a sticky drop-down list of organisms, including 'all' 
	void writeOrganismOptions(String prevOrg) throws SQLException
	{
		String orgQ = DBQuery.getOrganismNamesQuery();

		// start of section of form for drop down list of organisms
		out.println("<select name=\"organism\">");
		
		// all motifs at top of menu
		if(prevOrg.equals(allOrgs))
		{
			out.print("<option selected=\"selected\">");			
		}
		else
		{
			out.print("<option>");
		}
		out.print(allOrgs);
		out.println("</option>");	
		
		//create statement, execute and put results in drop-down
		Statement stmt = conn.createStatement();
		ResultSet resSet = stmt.executeQuery(orgQ);
		while (resSet.next())
		{
			String nextOrg = resSet.getString(1);
			out.print("<option value=\"" + nextOrg + "\" ");
			
			if(nextOrg.equals(prevOrg))
			{
				out.print("selected=\"selected\">");
			}
			else
			{
				out.print(">");
			}
			out.print(nextOrg);
			out.println("</option>");
		}
		
		// end of section of form for drop down list of organisms
		out.println("</select>");      
	}	
	
	// Utility method to get organism three-letter ID corresponding to organism name
	String getOrgCode(String organism) throws SQLException
	{
		ParamQuery codeQuery =DBQuery.getParamQuery("ORGANISM_ID");	
		codeQuery.setPrepStatement(conn);
		PreparedStatement prepStat = codeQuery.getPrepStatement();
		prepStat.setString(1, organism);
		ResultSet rSet = prepStat.executeQuery();
		rSet.next();	
		return rSet.getString(1);
	}
  		  		
  	// runs query to load Map array with details of all Maps present in a particular organism
	private void initializeMaps(String organism) throws SQLException  
	{
		boolean atStart = true;
		String currentMapName = "";
		Map currMap = null;
		ParamQuery mapQuery = null;
		
		mapSize = 0;				// in case second query?

		if(organism.equals(allOrgs))
		{
			mapQuery = DBQuery.getParamQuery("THEORO_METABOLITES_ALL");
		}
		else
		{
			mapQuery = DBQuery.getParamQuery("THEORO_METABOLITES_ORG");
		}
		mapQuery.setPrepStatement(conn);

		PreparedStatement prepStat = mapQuery.getPrepStatement();
		if(!organism.equals(allOrgs))
		{
			prepStat.setString(1, organism); 
		}
		ResultSet resSet = prepStat.executeQuery();

		// Analyse result set
		while (resSet.next()) // moves to next row while rows remain
		{
			String mapName = resSet.getString("MapName");
			String mapID = resSet.getString("MapID");
			// check if new map
			if (atStart) 
			{
				currentMapName = mapName; // set current initially
				atStart = false;
				// Construct new Map object and add to array of Maps
				currMap = new Map(mapName, mapID);
				mapSet[mapSize] = currMap;
				mapSize++;
				// Create Metabolite and add to Map object
				Metabolite nextMetab = new Metabolite(resSet.getString("CompoundName"), 
						resSet.getString("Formula"), resSet.getInt("FormDegen"), 
						resSet.getString("CompoundID"), resSet.getDouble("MIM"), 
						resSet.getString("GlyphShape"), resSet.getInt("GlyphX"), resSet.getInt("GlyphY"),
						resSet.getInt("GlyphWidth"), resSet.getInt("GlyphHeight"),
						numConditions);
				if(settings != null)
				{
					nextMetab.setSettings(settings);
				}

				// set metacycUID if not null
				String metacycUID = resSet.getString("MetacycUID");
				if(metacycUID != null)
				{
					nextMetab.setMetacycUID(metacycUID);
				}						
						
				currMap.addMetabolite(nextMetab);
			} 
			else if (mapName.equals(currentMapName)) 
			{
				// Create Metabolite and add to current Map object
				Metabolite nextMetab = new Metabolite(resSet.getString("CompoundName"), 
						resSet.getString("Formula"), resSet.getInt("FormDegen"), 
						resSet.getString("CompoundID"), resSet.getDouble("MIM"), 
						resSet.getString("GlyphShape"), resSet.getInt("GlyphX"), resSet.getInt("GlyphY"),
						resSet.getInt("GlyphWidth"), resSet.getInt("GlyphHeight"),						 
						numConditions);
				if(settings != null)
				{
					nextMetab.setSettings(settings);
				}
							
				// set metacycUID if not null
				String metacycUID = resSet.getString("MetacycUID");
				if(metacycUID != null)
				{
					nextMetab.setMetacycUID(metacycUID);
				}
				
				currMap.addMetabolite(nextMetab);
			} 
			else 
			{
				currentMapName = mapName;
				currMap = new Map(mapName, mapID);
				// Add to array of Maps
				mapSet[mapSize] = currMap;
				mapSize++;
				// Create Metabolite and add to Map object
				Metabolite nextMetab = new Metabolite(resSet.getString("CompoundName"), 
						resSet.getString("Formula"), resSet.getInt("FormDegen"), 
						resSet.getString("CompoundID"), resSet.getDouble("MIM"), 
						resSet.getString("GlyphShape"), resSet.getInt("GlyphX"), resSet.getInt("GlyphY"),
						resSet.getInt("GlyphWidth"), resSet.getInt("GlyphHeight"),
						numConditions);
				if(settings != null)
				{
					nextMetab.setSettings(settings);
				}						
				// set metacycUID if not null
				String metacycUID = resSet.getString("MetacycUID");
				if(metacycUID != null)
				{
					nextMetab.setMetacycUID(metacycUID);
				}
				
				currMap.addMetabolite(nextMetab);
			}
		}
	}
  		  				
	// sorts Maps in mapSet by order of unique occupancy (absolute, rather than %, which might be better)
	// If occupancy is same, sort by map name
	public void sortMaps() 
	{
		Map smallMap = new Map();

		for (int i = 0; i < mapSize - 1; i++) 
		{
			if (mapSet[i] != null) 
			{
				smallMap = mapSet[i];
				for (int j = i + 1; j < mapSize; j++) 
				{
					if (
						mapSet[j] != null && 
							(
								smallMap.getUniquePresent() < mapSet[j].getUniquePresent() ||
								( 
										smallMap.getUniquePresent() == mapSet[j].getUniquePresent() && 
										mapSet[j].getMapName().compareTo(smallMap.getMapName()) < 1 
								)
							) 
						) 
					{
						smallMap = mapSet[j];
						mapSet[j] = mapSet[i];
						mapSet[i] = smallMap;
					}
				}
			}
		}
	}
	
	// resets 'present' boolean to false for all Peaks - needed if reanalyse same expt
	public void resetPeaks()
	{
		for(int i=0; i<peakListSize; i++)
		{
			peakList[i].setPresent(false);
			peakList[i].clearFormulae();
		}
	}
	
	// sorts peaks by ascending MZ value
	public void sortPeaks()
	{
		Peak lowest;		// holder for Peak with lowest value
		int lowestPos;
		
		for (int i=0; i<peakListSize-1; i++)	// do series of runs
		{
			for (int j=i+1; j<peakListSize; j++)	// for ea run process list
			{
				lowest = peakList[i];	// first of unsorted assigned to lowest
				lowestPos = i;
				
				if(lowest.getMZ() > peakList[j].getMZ())
				{
					lowestPos = j;
				}
				
				lowest = peakList[lowestPos];
				peakList[lowestPos] = peakList[i]; 		// shift current first
				peakList[i] = lowest;					// replace
			}
		}
	}
	
	// resets 'present' boolean to false for all Compounds - is this needed???
	public void resetCompounds()
	{
		for(int i=0; i<cpdListSize; i++)
		{
			cpdList[i].setPresent(false);
		}
	}
	
	// looks for single quote in String and replaces it by \'
	public String escapeQuote(String name)
	{
		if(name.indexOf("'") != 0)
		{
			StringBuffer sb = new StringBuffer("");
			for (int i=0; i<name.length();i++)
			{
				char c = name.charAt(i);
				if(c == '\'')
				{
					sb.append( "\\" + c);
				}
				else
				{
					sb.append(c);
				}
			}
			return sb.toString();		
		}
		else
		{
			return name;
		}
	}

	  		
	// Reads a file into a String - typically a file in the same directory, e.g. "htmlText/mypage"
	// This allows one to write and edit external blocks of html without recompiling etc.
	public String readHTML(String path)
	{
		String outString;
		InputStream stream = getClass().getResourceAsStream(path);
		if (stream !=null)
		{
			try
			{
				byte [] b = new byte[8092];
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				int i = 0;
				while( (i=stream.read(b)) > 0)
				{
					out.write(b, 0, i);
				}
				stream.close();
				
				outString = out.toString();	
			}
			catch (IOException x)
			{
				outString = "Text Misread. Please notify the site owner.";
			}
		}
		else
		{
			outString = "Text Misread. Please notify the site owner.";		
		}
		return outString;
	}	
	
	// sets U of G proxy for server for external calls
    public void setUniProxy(String proxyHost, String proxyPort, boolean proxy)
    {
    	if(proxy)
    	{
	        System.getProperties().put("proxySet", "true");
	        System.getProperties().put("proxyHost", proxyHost);
	        System.getProperties().put("proxyPort", proxyPort);
        }
    }
}
