/*
Class holds details for an Experimental peak.
DPL 01.08.2011
 */

public class Peak 
{
	double mz;				// m/z (mass divided by charge) corresponding to peak	
	boolean present;		// whether correlated to a metabolite mass in Kegg DB
	
	String[] foundFormulae;	// array of formulae of metabolites that are present
	int formulaeLen = 20;	// initial length
	int formulaeSize = 0;	// initial occupancy
	
	MetAdd[] metAdds;		// array of objects composed of found metabolites and adduct
	int metaddLen = 20;		// initial length
	int metaddSize = 0;		// initial occupancy
	
	int len = 0;			// length of condition array
	int[] values;			// array of values
	int[] errors;			// array of errors

	public Peak(double mz, int len) 
	{
		this.mz = mz;		
		present = false;
		
		values = new int[len];
		errors = new int[len];
		// only use one of these - could have boolean I suppose
		foundFormulae = new String[formulaeLen];
		metAdds = new MetAdd[metaddLen];
	}
	
	// -- methods to populate arrays
	
	public void addValue(int val, int pos)
	{
		values[pos] = val;
	}
	
	public void addError(int error, int pos)
	{
		errors[pos] = error;
	}	
	
	// -- found formulae methods ---//
	
	public void addFormula(Metabolite metab, Adduct add)	
	{
		// check for duplicates first
		for(int i=0; i<metaddSize; i++)
		{
			if(metab.getFormula().equals(metAdds[i].getMetabolite().getFormula()))
			{
				return;
			}
		}		
		// expand array if necessary
		if(metaddSize >= metaddLen)
		{
			metaddLen = 2 * metaddLen;
			MetAdd[] newList = new MetAdd[metaddLen];
			System.arraycopy (metAdds, 0, newList, 0, metAdds.length);
			metAdds = newList;
		}
		metAdds[metaddSize] = new MetAdd(metab,add);
		metaddSize++;		
	}
	
	public void addFormula(String formula)	
	{
		// check for duplicates first
		for(int i=0; i<formulaeSize; i++)
		{
			if(formula.equals(foundFormulae[i]))
			{
				return;
			}
		}					
		// expand array if necessary
		if(formulaeSize >= formulaeLen)
		{
			formulaeLen = 2 * formulaeLen;
			String[] newList = new String[formulaeLen];
			System.arraycopy (foundFormulae, 0, newList, 0, foundFormulae.length);
			foundFormulae = newList;
		}		
		foundFormulae[formulaeSize] = formula;
		formulaeSize++;		
	}
	
	public String getFormula(int i)
	{
		return foundFormulae[i];	
	}
	
	public MetAdd getMetAdd(int i)
	{
		return metAdds[i];	
	}
	
	public int getFormulaeSize()
	{
		return formulaeSize;
	}
	
	public int getMetaddSize()
	{
		return metaddSize;
	}
	
	public void clearFormulae()
	{
		for(int i=0; i<formulaeSize; i++)
		{
			foundFormulae[i] = null;
		} 
		formulaeSize = 0;
		
		for(int i=0; i<metaddSize; i++)
		{
			metAdds[i] = null;
		} 
		metaddSize = 0;		
	}

	// -- accessor methods ---//

	public void setPresent(boolean found) 
	{
		present = found;
	}
	
	public double getMZ()
	{
		return mz;
	}

	public boolean isPresent() 
	{
		return present;
	}
	
	public int[] getValues()
	{
		return values;
	}
	
	public int[] getErrors()
	{
		return errors;
	}
	
	public int getValue(int pos)
	{
		return values[pos];
	}
	
	public int getError(int pos)
	{
		return errors[pos];
	}
	
	// inner class
	class MetAdd
	{
		Metabolite met;
		Adduct add;		
		public MetAdd(Metabolite met, Adduct add)
		{
			this.met = met;
			this.add = add;
		}
		public Metabolite getMetabolite()
		{
			return met;
		}
		public Adduct getAdduct()
		{
			return add;
		}
	}
	
}