#!/usr/bin/perl -wT
use strict;
use CGI qw(:standard);
use DBI;							# module to connect to DB

# ----------------------------- formula.cgi ----------------------------------
# CGI script which receives details of a Kegg compound,
# links to the graphic of the structure, 
# and lists other compounds of same formula with Kegg links
# Author: David Leader 03.05.10
# ------------------------------------------------------------------------

	               	# ----- Parmaters ---- #
	               
my $id = param('id');					# compound Kegg ID
my $name = param('name');				# compound Kegg name
my $formula = param('formula');			# formula (Kegg style)
my $degeneracy = param('degeneracy');	# extent formula is degenerate
my $host = param('host');				# e.g. 127.0.0.1 or motif.gla.ac.uk

	               # ----- DB variables ---- #
	
my $server = $host;
my $db = "Metabolism";				# Name of DB
my $username = "gordon";			# Select only username
my $password = "brown";				# Select only password
my $dbh;							# DB handle

	           # ----- Prepare and execute query ---- #
$dbh = DBI->connect("dbi:mysql:$db:$server",$username,$password);

my $query = "SELECT CompoundID, CompoundName FROM Metabolite WHERE Formula = \"$formula\"";	# SQL query

my $sth = $dbh->prepare($query);			# prepare query
$sth->execute();							# execute query

my @data;									# Array for query results
my $i = 0;									# counter
while (my $row = $sth->fetchrow_arrayref)	# build array
{
		$data[$i] = join("\t", @$row);		# separate fields with tabs
		$i += 1;	
}

$dbh->disconnect;

my $fformula = &formatFormula($formula);	# Format formula for display

print header;

# HTML head etc
print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n";
print "<html>\n<head>\n";
print "<title>Kegg Metabolite $id</title>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">\n";
print "<link rel=\"Stylesheet\" href=\"http://$host/scripts/formula.css\" type=\"text/css\">\n";
print "</head>\n<body>\n";

print "<h1>$name ($fformula)</h1>";

print "<div style=\"text-align:center;\"><img src=\"http://www.genome.jp/Fig/compound/$id.gif\"></div>";

print "<h2>Kegg map metabolites with formula  $fformula</h2>";
print "<div class=\"results\">";
foreach my $line(@data)
{
	my @lineArray = split(/\t/, $line);
	print "<a href=\"http://www.genome.jp/dbget-bin/www_bget?compound+$lineArray[0]\">$lineArray[0]</a>: ";			# id
	print "$lineArray[1]<br>\n";				# name
}
print "</div>";

if($degeneracy == 1)
{
	print "<p>(Metabolite ID links to full Kegg listing.)</p>";
}
else
{
	print "<p>(Metabolite IDs link to full Kegg listing.)</p>";
}

# Finish off HTML
print "\n</body>\n</html>";

# --- Subroutine to put html subscripts on formula (RE by James Goodlet)
sub formatFormula()
{
	my ($formatted) =  @_;
	$formatted =~ s/([[:alpha:]]+)([[:digit:]]+)/$1<sub>$2<\/sub>/g; 
	return $formatted;
}
