/*
	Entry Class for MS Comparison Servlet
	DP Leader 05.05.10
*/		

import java.io.*;
import javax.imageio.*;	
import java.awt.image.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class MSCompare extends HttpServlet
{			
	// Parameters
	String metabolite;		// name of metabolite
	int numConditions;		// number of conditions
	String[] names;			// array of names of conditions
	int[] values;		// array of condition values
	int[] errors;		// array of error values
	
	public void doGet(HttpServletRequest req, HttpServletResponse res)
                               throws ServletException, IOException 
	{			
		System.setProperty("java.awt.headless", "true");																	
		parseParams(req);
		CreateImage ci = new CreateImage(metabolite, numConditions, names, values, errors);
		BufferedImage buffImg = ci.getBufferedImage();
		
		res.setContentType("image/png");
		OutputStream os = res.getOutputStream();
		ImageIO.write(buffImg, "png", os);
		os.close();
	}							
	
	// parses parameters																							
	private void parseParams(HttpServletRequest req)
	{
		metabolite = req.getParameter("metabolite");
		
		try
		{		
			numConditions = Integer.parseInt(req.getParameter("numConditions"));
		}
		catch (NumberFormatException nfe)
		{
			System.out.println(nfe.toString());
		}
				
		String condNames = (String) req.getParameter("condNames");
		names = condNames.split(",");
		
		String condValues = (String) req.getParameter("condValues");
		String[] valueString = condValues.split(",");
		try
		{
			values = intArrayfromString(valueString, numConditions);
		}
		catch (NumberFormatException nfe)
		{
			System.out.println(nfe.toString());
		}
		
		String condErrors = (String) req.getParameter("condErrors");
		String[] errorString = condErrors.split(",");
		try
		{
			errors = intArrayfromString(errorString, numConditions);
		}
		catch (NumberFormatException nfe)
		{
			System.out.println(nfe.toString());
		}
	}
	
	// utility method to obtain double array from string array
	private int[] intArrayfromString(String[] stringArray, int size) throws NumberFormatException
	{
		int[] intArray = new int[size];	
		for (int i=0; i<size; i++)
		{
			intArray[i] = Integer.parseInt(stringArray[i]);
		}		
		return intArray;
	}

}
